@@ -103,12 +103,14 @@ type_names['signed short'] = TYPE_SHORT
 type_names['unsigned short'] = TYPE_USHORT
 type_names['int'] = TYPE_INT
 type_names['signed int'] = TYPE_INT
+type_names['unsigned short int'] = TYPE_USHORT
 type_names['signed'] = TYPE_INT
 type_names['unsigned int'] = TYPE_UINT
 type_names['unsigned'] = TYPE_UINT
 type_names['long'] = TYPE_LONG
 type_names['signed long'] = TYPE_LONG
 type_names['unsigned long'] = TYPE_ULONG
+type_names['unsigned long int'] = TYPE_ULONG
 type_names['float'] = TYPE_FLOAT
 type_names['double'] = TYPE_DOUBLE
 type_names['char*'] = TYPE_STRING
@@ -365,6 +367,7 @@ class Parameter(TypeContainer):
         else:
             self.direction = PARAM_DIRECTION_IN
 
+        self.caller_allocates = False
         self.allow_none = allow_none
         self.scope = scope
         self.closure_index = -1
